"""
parser.http.plotParser module (imdb package).

This module provides the HTMLPlotParser class (and the plot_parser
instance), used to parse the "plot summary" IMDb page on the
akas.imdb.com server.
E.g., for Brian De Palma's "The Untouchables", the referred
page would be 'http://akas.imdb.com/title/tt0094226/plotsummary'

Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from htmllib import HTMLParser
from formatter import NullFormatter

from imdb._exceptions import IMDbParserError
from imdb.parser.http.utils import get_attr_value


# XXX: this class inherits from HTMLParser; see the documentation for
#      the "htmllib" and "sgmllib" modules.
class HTMLPlotParser(HTMLParser):
    """Parser for the "plot summary" page of a given movie.
    The page should be provided as a string, as taken from
    the akas.imdb.com server.  The final result will be a
    dictionary, with a 'plot' key, containing a list
    of string with the structure: 'summary_author <author@email>::summary'.
    
    Example:
        pparser = HTMLPlotParser()
        result = pparser.parse(plot_summary_html_string)
    """
    def __init__(self, formatter=NullFormatter(), verbose=0):
        self.__plot_data = {}
        HTMLParser.__init__(self, formatter, verbose)
        self.entitydefs['nbsp'] = ' '

    def reset(self):
        """Reset the parser."""
        HTMLParser.reset(self)
        self.__plot_data.clear()
        self.__is_plot = 0
        self.__plot = ''
        self.__last_plot = ''
        self.__is_plot_writer = 0
        self.__plot_writer = ''

    def get_plot_data(self):
        """Return the dictionary with the 'plot' key."""
        return self.__plot_data

    def start_p(self, attrs):
        pclass = get_attr_value(attrs, 'class')
        if pclass == 'plotpar':
            self.__is_plot = 1

    def end_p(self):
        if self.__is_plot:
            # Store the plot in the self.__last_plot variable until
            # the parser will read the name of the author.
            self.__last_plot = self.__plot
            self.__is_plot = 0
            self.__plot = ''

    def start_a(self, attrs):
        link = get_attr_value(attrs, 'href')
        # The next data is the name of the author.
        if link and link.lower().startswith('/searchplotwriters'):
            self.__is_plot_writer = 1

    def end_a(self):
        # We've read the name of an author and the summary he wrote;
        # store everything in __plot_data.
        if self.__is_plot_writer and self.__last_plot:
            writer = self.__plot_writer.strip()
            # Replace funny email separators.
            writer = writer.replace('{', '<').replace('}', '>')
            plot = self.__last_plot.strip()
            if not self.__plot_data.has_key('plot'):
                self.__plot_data['plot'] = []
            self.__plot_data['plot'].append(writer + '::' + plot)
            self.__is_plot_writer = 0
            self.__plot_writer = ''
            self.__last_plot = ''
            
    def handle_data(self, data):
        # Store text for plots and authors.
        if self.__is_plot:
            self.__plot += data
        if self.__is_plot_writer:
            self.__plot_writer += data

    def error(self, message):
        raise IMDbParserError, 'SGML parser error: "%s"' % str(message)

    def parse(self, plot_html):
        """Return a dictionary with a 'plot' key, from the given html string."""
        self.reset()
        self.feed(plot_html)
        return self.get_plot_data()


# The used instance.
plot_parser = HTMLPlotParser()


